@@ -1,8 +1,6 @@
 import Text.Pandoc
-import Text.Pandoc.Shared (normalize)
 import Criterion.Main
 import Criterion.Config
-import Text.JSON.Generic
 import System.Environment (getArgs)
 import Data.Monoid
 
@@ -15,8 +13,7 @@ readerBench doc (name, reader) =
                      _ -> error $ "Could not find writer for " ++ name
       inp = writer def{ writerWrapText = True } doc
       -- we compute the length to force full evaluation
-      getLength (Pandoc (Meta a b c) d) =
-            length a + length b + length c + length d
+      getLength (Pandoc (Meta _) d) = length d
   in  bench (name ++ " reader") $ whnfIO $ getLength `fmap`
       (reader def{ readerSmart = True }) inp
 
@@ -26,11 +23,6 @@ writerBench :: Pandoc
 writerBench doc (name, writer) = bench (name ++ " writer") $ nf
     (writer def{ writerWrapText = True }) doc
 
-normalizeBench :: Pandoc -> [Benchmark]
-normalizeBench doc = [ bench "normalize - with" $ nf (encodeJSON . normalize) doc
-                     , bench "normalize - without" $ nf encodeJSON doc
-                     ]
-
 main :: IO ()
 main = do
   args <- getArgs
@@ -39,8 +31,9 @@ main = do
   inp2 <- readFile "tests/testsuite.txt"
   let opts = def{ readerSmart = True }
   let doc = readMarkdown opts $ inp ++ unlines (drop 3 $ lines inp2)
-  let readerBs = map (readerBench doc) readers
+  let readerBs = map (readerBench doc)
+                 $ filter (\(n,_) -> n /="haddock") readers
   let writers' = [(n,w) | (n, PureStringWriter w) <- writers]
   defaultMainWith conf (return ()) $
-    map (writerBench doc) writers' ++ readerBs ++ normalizeBench doc
+    map (writerBench doc) writers' ++ readerBs
 
