@@ -48,13 +48,13 @@ import Data.Set (Set)
 import qualified Data.Set as Set
 import Data.Default
 import Text.Pandoc.Highlighting (Style, pygments)
-import qualified Text.CSL as CSL
 
 -- | Individually selectable syntax extensions.
 data Extension =
       Ext_footnotes           -- ^ Pandoc/PHP/MMD style footnotes
     | Ext_inline_notes        -- ^ Pandoc-style inline notes
     | Ext_pandoc_title_block  -- ^ Pandoc title block
+    | Ext_yaml_metadata_block -- ^ YAML metadata block
     | Ext_mmd_title_block     -- ^ Multimarkdown metadata block
     | Ext_table_captions      -- ^ Pandoc-style table captions
     | Ext_implicit_figures    -- ^ A paragraph with just an image is a figure
@@ -80,6 +80,7 @@ data Extension =
     | Ext_link_attributes     -- ^ MMD style reference link attributes
     | Ext_autolink_bare_uris  -- ^ Make all absolute URIs into links
     | Ext_fancy_lists         -- ^ Enable fancy list numbers and delimiters
+    | Ext_lists_without_preceding_blankline -- ^ Allow lists without preceding blank
     | Ext_startnum            -- ^ Make start number of ordered list significant
     | Ext_definition_lists    -- ^ Definition lists as in pandoc, mmd, php
     | Ext_example_lists       -- ^ Markdown-style numbered examples
@@ -91,9 +92,11 @@ data Extension =
     | Ext_superscript         -- ^ Superscript using ^this^ syntax
     | Ext_subscript           -- ^ Subscript using ~this~ syntax
     | Ext_hard_line_breaks    -- ^ All newlines become hard line breaks
+    | Ext_ignore_line_breaks  -- ^ Newlines in paragraphs are ignored
     | Ext_literate_haskell    -- ^ Enable literate Haskell conventions
     | Ext_abbreviations       -- ^ PHP markdown extra abbreviation definitions
     | Ext_auto_identifiers    -- ^ Automatic identifiers for headers
+    | Ext_ascii_identifiers   -- ^ ascii-only identifiers for headers
     | Ext_header_attributes   -- ^ Explicit header attributes {#id .class k=v}
     | Ext_mmd_header_identifiers -- ^ Multimarkdown style header identifiers [myid]
     | Ext_implicit_header_references -- ^ Implicit reference links for headers
@@ -105,6 +108,7 @@ pandocExtensions = Set.fromList
   [ Ext_footnotes
   , Ext_inline_notes
   , Ext_pandoc_title_block
+  , Ext_yaml_metadata_block
   , Ext_table_captions
   , Ext_implicit_figures
   , Ext_simple_tables
@@ -159,11 +163,14 @@ githubMarkdownExtensions = Set.fromList
   , Ext_tex_math_single_backslash
   , Ext_fenced_code_blocks
   , Ext_fenced_code_attributes
+  , Ext_auto_identifiers
+  , Ext_ascii_identifiers
   , Ext_backtick_code_blocks
   , Ext_autolink_bare_uris
   , Ext_intraword_underscores
   , Ext_strikeout
   , Ext_hard_line_breaks
+  , Ext_lists_without_preceding_blankline
   ]
 
 multimarkdownExtensions :: Set Extension
@@ -199,8 +206,6 @@ data ReaderOptions = ReaderOptions{
        , readerOldDashes       :: Bool -- ^ Use pandoc <= 1.8.2.1 behavior
                                        --   in parsing dashes; -- is em-dash;
                                        --   - before numerial is en-dash
-       , readerReferences      :: [CSL.Reference]  -- ^ Bibliographic references
-       , readerCitationStyle   :: Maybe CSL.Style -- ^ Citation style
        , readerApplyMacros     :: Bool -- ^ Apply macros to TeX math
        , readerIndentedCodeClasses :: [String] -- ^ Default classes for
                                        -- indented code blocks
@@ -217,8 +222,6 @@ instance Default ReaderOptions
                , readerColumns               = 80
                , readerTabStop               = 4
                , readerOldDashes             = False
-               , readerReferences            = []
-               , readerCitationStyle         = Nothing
                , readerApplyMacros           = True
                , readerIndentedCodeClasses   = []
                , readerDefaultImageExtension = ""
@@ -255,6 +258,7 @@ data HTMLSlideVariant = S5Slides
                       | SlidySlides
                       | SlideousSlides
                       | DZSlides
+                      | RevealJsSlides
                       | NoSlides
                       deriving (Show, Read, Eq)
 
@@ -279,10 +283,9 @@ data WriterOptions = WriterOptions
   , writerEmailObfuscation :: ObfuscationMethod -- ^ How to obfuscate emails
   , writerIdentifierPrefix :: String -- ^ Prefix for section & note ids in HTML
                                      -- and for footnote marks in markdown
-  , writerSourceDirectory  :: FilePath -- ^ Directory path of 1st source file
+  , writerSourceURL        :: Maybe String  -- ^ Absolute URL + directory of 1st source file
   , writerUserDataDir      :: Maybe FilePath -- ^ Path of user data directory
   , writerCiteMethod       :: CiteMethod -- ^ How to print cites
-  , writerBiblioFiles      :: [FilePath] -- ^ Biblio files to use for citations
   , writerHtml5            :: Bool       -- ^ Produce HTML5
   , writerHtmlQTags        :: Bool       -- ^ Use @<q>@ tags for quotes in HTML
   , writerBeamer           :: Bool       -- ^ Produce beamer LaTeX slide show
@@ -322,10 +325,9 @@ instance Default WriterOptions where
                       , writerColumns          = 72
                       , writerEmailObfuscation = JavascriptObfuscation
                       , writerIdentifierPrefix = ""
-                      , writerSourceDirectory  = "."
+                      , writerSourceURL        = Nothing
                       , writerUserDataDir      = Nothing
                       , writerCiteMethod       = Citeproc
-                      , writerBiblioFiles      = []
                       , writerHtml5            = False
                       , writerHtmlQTags        = False
                       , writerBeamer           = False
