@@ -52,11 +52,13 @@ TODO : refactor common patterns across readers :
 module Text.Pandoc.Readers.Textile ( readTextile) where
 
 import Text.Pandoc.Definition
+import qualified Text.Pandoc.Builder as B
 import Text.Pandoc.Shared
 import Text.Pandoc.Options
 import Text.Pandoc.Parsing
 import Text.Pandoc.Readers.HTML ( htmlTag, isInlineTag, isBlockTag )
 import Text.Pandoc.Readers.LaTeX ( rawLaTeXInline, rawLaTeXBlock )
+import Text.HTML.TagSoup (parseTags, innerText, fromAttrib, Tag(..))
 import Text.HTML.TagSoup.Match
 import Data.List ( intercalate )
 import Data.Char ( digitToInt, isUpper )
@@ -93,7 +95,7 @@ parseTextile = do
   updateState $ \s -> s { stateNotes = reverse reversedNotes }
   -- now parse it for real...
   blocks <- parseBlocks
-  return $ Pandoc (Meta [] [] []) blocks -- FIXME
+  return $ Pandoc nullMeta blocks -- FIXME
 
 noteMarker :: Parser [Char] ParserState [Char]
 noteMarker = skipMany spaceChar >> string "fn" >> manyTill digit (char '.')
@@ -152,8 +154,10 @@ codeBlockBc = try $ do
 -- | Code Blocks in Textile are between <pre> and </pre>
 codeBlockPre :: Parser [Char] ParserState Block
 codeBlockPre = try $ do
-  htmlTag (tagOpen (=="pre") null)
-  result' <- manyTill anyChar (try $ htmlTag (tagClose (=="pre")) >> blockBreak)
+  (t@(TagOpen _ attrs),_) <- htmlTag (tagOpen (=="pre") (const True))
+  result' <- (innerText . parseTags) `fmap` -- remove internal tags
+               manyTill anyChar (htmlTag (tagClose (=="pre")))
+  optional blanklines
   -- drop leading newline if any
   let result'' = case result' of
                       '\n':xs -> xs
@@ -162,7 +166,10 @@ codeBlockPre = try $ do
   let result''' = case reverse result'' of
                        '\n':_ -> init result''
                        _      -> result''
-  return $ CodeBlock ("",[],[]) result'''
+  let classes = words $ fromAttrib "class" t
+  let ident = fromAttrib "id" t
+  let kvs = [(k,v) | (k,v) <- attrs, k /= "id" && k /= "class"]
+  return $ CodeBlock (ident,classes,kvs) result'''
 
 -- | Header of the form "hN. content" with N in 1..6
 header :: Parser [Char] ParserState Block
@@ -173,7 +180,8 @@ header = try $ do
   char '.'
   whitespace
   name <- normalizeSpaces <$> manyTill inline blockBreak
-  return $ Header level attr name
+  attr' <- registerHeader attr (B.fromList name)
+  return $ Header level attr' name
 
 -- | Blockquote of the form "bq. content"
 blockQuote :: Parser [Char] ParserState Block
@@ -275,7 +283,7 @@ definitionListItem = try $ do
 -- blocks support, we have to lookAhead for a rawHtmlBlock.
 blockBreak :: Parser [Char] ParserState ()
 blockBreak = try (newline >> blanklines >> return ()) <|>
-              (lookAhead rawHtmlBlock >> return ())
+             try (optional spaces >> lookAhead rawHtmlBlock >> return ())
 
 -- raw content
 
@@ -284,13 +292,13 @@ rawHtmlBlock :: Parser [Char] ParserState Block
 rawHtmlBlock = try $ do
   (_,b) <- htmlTag isBlockTag
   optional blanklines
-  return $ RawBlock "html" b
+  return $ RawBlock (Format "html") b
 
 -- | Raw block of LaTeX content
 rawLaTeXBlock' :: Parser [Char] ParserState Block
 rawLaTeXBlock' = do
   guardEnabled Ext_raw_tex
-  RawBlock "latex" <$> (rawLaTeXBlock <* spaces)
+  RawBlock (Format "latex") <$> (rawLaTeXBlock <* spaces)
 
 
 -- | In textile, paragraphs are separated by blank lines.
@@ -374,6 +382,7 @@ inlineParsers = [ str
                 , link
                 , image
                 , mark
+                , (Str . (:[])) <$> characterReference
                 , smartPunctuation inline
                 , symbol
                 ]
@@ -424,7 +433,7 @@ note = try $ do
 
 -- | Special chars
 markupChars :: [Char]
-markupChars = "\\*#_@~-+^|%=[]"
+markupChars = "\\*#_@~-+^|%=[]&"
 
 -- | Break strings on following chars. Space tab and newline break for
 --  inlines breaking. Open paren breaks for mark. Quote, dash and dot
@@ -480,7 +489,7 @@ endline = try $ do
   return LineBreak
 
 rawHtmlInline :: Parser [Char] ParserState Inline
-rawHtmlInline = RawInline "html" . snd <$> htmlTag isInlineTag
+rawHtmlInline = RawInline (Format "html") . snd <$> htmlTag isInlineTag
 
 -- | Raw LaTeX Inline
 rawLaTeXInline' :: Parser [Char] ParserState Inline
