@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 {-
 Copyright (C) 2010 John MacFarlane <jgm@berkeley.edu>
 
@@ -47,8 +48,12 @@ where
 
 import System.IO hiding (readFile, writeFile, getContents,
                           putStr, putStrLn, hPutStr, hPutStrLn, hGetContents)
+#if MIN_VERSION_base(4,6,0)
+import Prelude hiding (readFile, writeFile, getContents, putStr, putStrLn)
+#else
 import Prelude hiding (readFile, writeFile, getContents, putStr, putStrLn,
                        catch)
+#endif
 import qualified System.IO as IO
 import qualified Data.ByteString.Char8 as B
 import qualified Data.ByteString.Lazy as BL
@@ -81,7 +86,7 @@ hPutStrLn :: Handle -> String -> IO ()
 hPutStrLn h s = hSetEncoding h utf8 >> IO.hPutStrLn h s
 
 hGetContents :: Handle -> IO String
-hGetContents = fmap toStringLazy . BL.hGetContents
+hGetContents = fmap toString . B.hGetContents
 -- hGetContents h = hSetEncoding h utf8_bom
 --                   >> hSetNewlineMode h universalNewlineMode
 --                   >> IO.hGetContents h
