@@ -144,8 +144,11 @@ pandocToHtml opts (Pandoc meta blocks) = do
                            MathJax url ->
                               H.script ! A.src (toValue url)
                                        ! A.type_ "text/javascript"
-                                       $ preEscapedString
-                                         "MathJax.Hub.Queue([\"Typeset\",MathJax.Hub]);"
+                                       $ case writerSlideVariant opts of
+                                              SlideousSlides ->
+                                                 preEscapedString
+                                                 "MathJax.Hub.Queue([\"Typeset\",MathJax.Hub]);"
+                                              _ -> mempty
                            JsMath (Just url) ->
                               H.script ! A.src (toValue url)
                                        ! A.type_ "text/javascript"
@@ -283,9 +286,8 @@ elementToHtml slideLevel opts (Sec level num (id',classes,keyvals) title' elemen
                         then filter isSec elements
                         else if slide
                                 then case splitBy isPause elements of
-                                          []   -> []
-                                          [x]  -> x
-                                          xs   -> concatMap inDiv xs
+                                          []     -> []
+                                          (x:xs) -> x ++ concatMap inDiv xs
                                 else elements
   let inNl x = mconcat $ nl opts : intersperse (nl opts) x ++ [nl opts]
   let classes' = ["titleslide" | titleSlide] ++ ["slide" | slide] ++
@@ -418,9 +420,16 @@ blockToHtml opts (Para [Image txt (s,'f':'i':'g':':':tit)]) = do
 blockToHtml opts (Para lst) = do
   contents <- inlineListToHtml opts lst
   return $ H.p contents
-blockToHtml opts (Div attr bs) = do
+blockToHtml opts (Div attr@(_,classes,_) bs) = do
   contents <- blockListToHtml opts bs
-  return $ addAttrs opts attr $ H.div $ nl opts >> contents >> nl opts
+  let contents' = nl opts >> contents >> nl opts
+  return $
+     if "notes" `elem` classes
+        then case writerSlideVariant opts of
+                  RevealJsSlides -> addAttrs opts attr $ H5.aside $ contents'
+                  NoSlides       -> addAttrs opts attr $ H.div $ contents'
+                  _              -> mempty
+        else addAttrs opts attr $ H.div $ contents'
 blockToHtml _ (RawBlock f str)
   | f == Format "html" = return $ preEscapedString str
   | otherwise          = return mempty
@@ -604,8 +613,22 @@ inlineToHtml opts inline =
     (Str str)        -> return $ strToHtml str
     (Space)          -> return $ strToHtml " "
     (LineBreak)      -> return $ if writerHtml5 opts then H5.br else H.br
-    (Span attr ils)  -> inlineListToHtml opts ils >>=
-                           return . addAttrs opts attr . H.span
+    (Span (id',classes,kvs) ils)
+                     -> inlineListToHtml opts ils >>=
+                           return . addAttrs opts attr' . H.span
+                        where attr' = (id',classes',kvs')
+                              classes' = filter (`notElem` ["csl-no-emph",
+                                              "csl-no-strong",
+                                              "csl-no-smallcaps"]) classes
+                              kvs' = if null styles
+                                        then kvs
+                                        else (("style", concat styles) : kvs)
+                              styles = ["font-style:normal;"
+                                         | "csl-no-emph" `elem` classes]
+                                    ++ ["font-weight:normal;"
+                                         | "csl-no-strong" `elem` classes]
+                                    ++ ["font-variant:normal;"
+                                         | "csl-no-smallcaps" `elem` classes]
     (Emph lst)       -> inlineListToHtml opts lst >>= return . H.em
     (Strong lst)     -> inlineListToHtml opts lst >>= return . H.strong
     (Code attr str)  -> case hlCode of
@@ -676,14 +699,14 @@ inlineToHtml opts inline =
                                         Right r -> return $ preEscapedString $
                                                     ppcElement conf r
                                         Left  _ -> inlineListToHtml opts
-                                                   (readTeXMath str) >>= return .
+                                                   (readTeXMath' t str) >>= return .
                                                      (H.span ! A.class_ "math")
                                MathJax _ -> return $ H.span ! A.class_ "math" $ toHtml $
                                   case t of
                                     InlineMath  -> "\\(" ++ str ++ "\\)"
                                     DisplayMath -> "\\[" ++ str ++ "\\]"
                                PlainMath -> do
-                                  x <- inlineListToHtml opts (readTeXMath str)
+                                  x <- inlineListToHtml opts (readTeXMath' t str)
                                   let m = H.span ! A.class_ "math" $ x
                                   let brtag = if writerHtml5 opts then H5.br else H.br
                                   return  $ case t of
@@ -748,11 +771,11 @@ inlineToHtml opts inline =
                                          writerIdentifierPrefix opts ++ "fn" ++ ref)
                                        ! A.class_ "footnoteRef"
                                        ! prefixedId opts ("fnref" ++ ref)
+                                       $ H.sup
                                        $ toHtml ref
-                        let link' = case writerEpubVersion opts of
-                                         Just EPUB3 -> link ! customAttribute "epub:type" "noteref"
-                                         _ -> link
-                        return $ H.sup $ link'
+                        return $ case writerEpubVersion opts of
+                                      Just EPUB3 -> link ! customAttribute "epub:type" "noteref"
+                                      _          -> link
     (Cite cits il)-> do contents <- inlineListToHtml opts il
                         let citationIds = unwords $ map citationId cits
                         let result = H.span ! A.class_ "citation" $ contents
