@@ -64,6 +64,7 @@ data WriterState =
                 , stInDefinition  :: Bool
                 , stTight         :: Bool
                 , stFirstPara     :: Bool
+                , stImageId       :: Int
                 }
 
 defaultWriterState :: WriterState
@@ -78,6 +79,7 @@ defaultWriterState =
                 , stInDefinition  = False
                 , stTight         = False
                 , stFirstPara     = False
+                , stImageId       = 1
                 }
 
 when :: Bool -> Doc -> Doc
@@ -373,23 +375,27 @@ inlineToOpenDocument o ils
     | Subscript   l <- ils = withTextStyle Sub    $ inlinesToOpenDocument o l
     | SmallCaps   l <- ils = withTextStyle SmallC $ inlinesToOpenDocument o l
     | Quoted    t l <- ils = inQuotes t <$> inlinesToOpenDocument o l
-    | Code      _ s <- ils = preformatted s
-    | Math      _ s <- ils = inlinesToOpenDocument o (readTeXMath s)
+    | Code      _ s <- ils = withTextStyle Pre $ inTextStyle $ preformatted s
+    | Math      t s <- ils = inlinesToOpenDocument o (readTeXMath' t s)
     | Cite      _ l <- ils = inlinesToOpenDocument o l
-    | RawInline f s <- ils = if f == "opendocument" || f == "html"
-                                then preformatted s
+    | RawInline f s <- ils = if f == "opendocument"
+                                then return $ preformatted s
                                 else return empty
     | Link  l (s,t) <- ils = mkLink s t <$> inlinesToOpenDocument o l
-    | Image _ (s,t) <- ils = return $ mkImg  s t
+    | Image _ (s,t) <- ils = mkImg  s t
     | Note        l <- ils = mkNote l
     | otherwise            = return empty
     where
-      preformatted = return . inSpanTags "Teletype" . handleSpaces . escapeStringForXML
+      preformatted s = handleSpaces $ escapeStringForXML s
       mkLink   s t = inTags False "text:a" [ ("xlink:type" , "simple")
                                            , ("xlink:href" , s       )
                                            , ("office:name", t       )
                                            ] . inSpanTags "Definition"
-      mkImg  s t   = inTags False "draw:frame" (attrsFromTitle t) $
+      mkImg  s t   = do
+               id' <- gets stImageId
+               modify (\st -> st{ stImageId = id' + 1 })
+               return $ inTags False "draw:frame"
+                        (("draw:name", "img" ++ show id'):attrsFromTitle t) $
                      selfClosingTag "draw:image" [ ("xlink:href"   , s       )
                                                  , ("xlink:type"   , "simple")
                                                  , ("xlink:show"   , "embed" )
@@ -524,7 +530,8 @@ paraTableStyles t s (a:xs)
                      [ ("fo:text-align", x)
                      , ("style:justify-single-word", "false")]
 
-data TextStyle = Italic | Bold | Strike | Sub | Sup | SmallC deriving ( Eq,Ord )
+data TextStyle = Italic | Bold | Strike | Sub | Sup | SmallC | Pre 
+               deriving ( Eq,Ord )
 
 textStyleAttr :: TextStyle -> [(String,String)]
 textStyleAttr s
@@ -538,5 +545,8 @@ textStyleAttr s
     | Sub    <- s = [("style:text-position"          ,"sub 58%"   )]
     | Sup    <- s = [("style:text-position"          ,"super 58%" )]
     | SmallC <- s = [("fo:font-variant"              ,"small-caps")]
+    | Pre    <- s = [("style:font-name"              ,"Courier New")
+                    ,("style:font-name-asian"        ,"Courier New")
+                    ,("style:font-name-complex"      ,"Courier New")]
     | otherwise   = []
 
