@@ -33,7 +33,9 @@ module Text.Pandoc.Writers.Textile ( writeTextile ) where
 import Text.Pandoc.Definition
 import Text.Pandoc.Options
 import Text.Pandoc.Shared
-import Text.Pandoc.Templates (renderTemplate)
+import Text.Pandoc.Pretty (render)
+import Text.Pandoc.Writers.Shared
+import Text.Pandoc.Templates (renderTemplate')
 import Text.Pandoc.XML ( escapeStringForXML )
 import Data.List ( intercalate )
 import Control.Monad.State
@@ -53,13 +55,15 @@ writeTextile opts document =
 
 -- | Return Textile representation of document.
 pandocToTextile :: WriterOptions -> Pandoc -> State WriterState String
-pandocToTextile opts (Pandoc _ blocks) = do
+pandocToTextile opts (Pandoc meta blocks) = do
+  metadata <- metaToJSON opts (blockListToTextile opts)
+                 (inlineListToTextile opts) meta
   body <- blockListToTextile opts blocks
   notes <- liftM (unlines . reverse . stNotes) get
   let main = body ++ if null notes then "" else ("\n\n" ++ notes)
-  let context = writerVariables opts ++ [ ("body", main) ]
+  let context = defField "body" main metadata
   if writerStandalone opts
-     then return $ renderTemplate context $ writerTemplate opts
+     then return $ renderTemplate' (writerTemplate opts) context
      else return main
 
 withUseTags :: State WriterState a -> State WriterState a
@@ -98,6 +102,12 @@ blockToTextile :: WriterOptions -- ^ Options
 
 blockToTextile _ Null = return ""
 
+blockToTextile opts (Div attr bs) = do
+  let startTag = render Nothing $ tagWithAttrs "div" attr
+  let endTag = "</div>"
+  contents <- blockListToTextile opts bs
+  return $ startTag ++ "\n\n" ++ contents ++ "\n\n" ++ endTag ++ "\n"
+
 blockToTextile opts (Plain inlines) =
   inlineListToTextile opts inlines
 
@@ -115,10 +125,9 @@ blockToTextile opts (Para inlines) = do
               then "<p>" ++ contents ++ "</p>"
               else contents ++ if null listLevel then "\n" else ""
 
-blockToTextile _ (RawBlock f str) =
-  if f == "html" || f == "textile"
-     then return str
-     else return ""
+blockToTextile _ (RawBlock f str)
+  | f == Format "html" || f == Format "textile" = return str
+  | otherwise                                   = return ""
 
 blockToTextile _ HorizontalRule = return "<hr />\n"
 
@@ -340,6 +349,9 @@ inlineListToTextile opts lst =
 -- | Convert Pandoc inline element to Textile.
 inlineToTextile :: WriterOptions -> Inline -> State WriterState String
 
+inlineToTextile opts (Span _ lst) =
+  inlineListToTextile opts lst
+
 inlineToTextile opts (Emph lst) = do
   contents <- inlineListToTextile opts lst
   return $ if '_' `elem` contents
@@ -392,10 +404,9 @@ inlineToTextile _ (Str str) = return $ escapeStringForTextile str
 inlineToTextile _ (Math _ str) =
   return $ "<span class=\"math\">" ++ escapeStringForXML str ++ "</math>"
 
-inlineToTextile _ (RawInline f str) =
-  if f == "html" || f == "textile"
-     then return str
-     else return ""
+inlineToTextile _ (RawInline f str)
+  | f == Format "html" || f == Format "textile" = return str
+  | otherwise                                   = return ""
 
 inlineToTextile _ (LineBreak) = return "\n"
 
